from .phq_qmnli import phq_qmnli_list
from .phq_qmlm import phq_qmlm_list
import os
import pandas as pd
from ...utils import QuestionnaireData

data = []

# Get the name of the parent directory of the current file directory
parent_directory_name = os.path.basename(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
category_name = parent_directory_name

def get_questionnaire_info(question_class):
    task_name = question_class.__bases__[0].__name__
    question_obj = question_class()
    questionnaire_name = question_obj._descriptor["Questionnair"] 
    return task_name, questionnaire_name


if phq_qmlm_list:
    question_class_nli = phq_qmlm_list[0]
    task_name, questionnaire_name = get_questionnaire_info(question_class_nli)
    for mlm_question in phq_qmlm_list:
        data.append((category_name, questionnaire_name, task_name, mlm_question))


if phq_qmnli_list:
    question_class_nli = phq_qmnli_list[0]
    task_name, questionnaire_name = get_questionnaire_info(question_class_nli)
    for nli_question in phq_qmnli_list:
        data.append((category_name, questionnaire_name, task_name, nli_question))

phq_questionnaire = QuestionnaireData(pd.DataFrame(data, columns=['category_name', 'questionnaire_name', 'questionnaire_task', 'question']))

__all__ = ['phq_questionnaire']


from qlatent.qmlm.qmlm import QMLM, SCALE, dict_pos_neg


frequency_weights:SCALE = {
    'never':-4,
    'very rarely':-3,
    'seldom':-2,
    'rarely':-2,
    'frequently':2,
    'often':2,
    'very frequently':3,
    'always':4,    
}


class PHQ9Q1(QMLM):

    emo_pos = ['unsatisfied', 'upset', 'annoyed']
    emo_neg = ['pleased', 'happy', 'satisfied']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I have {frequency} been {index} in doing things.",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":1,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Little interest or pleasure in doing things'
            },
            **kwargs
        )
        


class PHQ9Q2(QMLM):

    emo_pos=['down', 'depressed', 'hopeless', 'sad']
    emo_neg=['happy', 'hopeful', 'cheerful']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs ):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I was {frequency} feeling {index}.",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":2,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Feeling down, depressed or hopeless'
            },
            **kwargs
        )
        


class PHQ9Q3(QMLM):

    emo_pos=['had trouble falling asleep', 'had trouble staying asleep']
    emo_neg=['been resting well', 'been sleeping well']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I have {frequency} {index}.",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":3,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Trouble falling asleep, staying asleep, or sleeping too much'
            },
            **kwargs
        )
        
        
class PHQ9Q4(QMLM):

    emo_pos=['tired', 'drained', 'fatigued']
    emo_neg=['energized', 'refreshed', 'lively']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I have {frequency} felt {index}.",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":4,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Feeling tired or having little energy'
            },
            **kwargs
        )
        
        


class PHQ9Q5(QMLM):

    emo_pos=['poor appetite', 'been overeating']
    emo_neg=['healthy appetite', 'satisfying appetite']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I {frequency} had a {index}.",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":5,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Poor appetite or overeating'
            },
            **kwargs
        )



class PHQ9Q6(QMLM):

    
    emo_pos=['a failure', 'a disappointment', 'letting myself down', 'letting my family down']
    emo_neg=['successful', 'lucky', 'confident']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I have {frequency} felt that I am {index}.",            
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":6,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Feeling bad about yourself - or that you’re a failure or have let yourself or your family down'
            },
            **kwargs
        )
        
    

class PHQ9Q7(QMLM):

    emo_pos=['have trouble concentrating', 'have difficulty focusing']
    emo_neg=['concentrate easily', 'focus effortlessly']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I {frequency} {index}.",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":7,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Trouble concentrating on things, such as reading the newspaper or watching television'
            },
            **kwargs
        )
        
        


class PHQ9Q8(QMLM):

    emo_pos=['fidgetly', 'slowly']
    emo_neg=['normally', 'naturally']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I have {frequency} moved or spoken {index}.",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":8,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Moving or speaking so slowly that other people could have noticed. Or, the opposite - being so fidgety or restless that you have been moving around a lot more than usual'
            },
            **kwargs
        )
        


class PHQ9Q9(QMLM):

    emo_pos=['suicidal', 'self destructive']
    emo_neg=['harmless', 'hopeful', 'positive']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "Over the last 2 weeks, I have {frequency} had {index} thoughts.",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"PHQ9",
              "Factor":"PHQ9",
              "Ordinal":9,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Thoughts that you would be better off dead or of hurting yourself in some way'
            },
            **kwargs
        )
        
        
phq_qmlm_list = [PHQ9Q1, PHQ9Q2, PHQ9Q3, PHQ9Q4, PHQ9Q5, PHQ9Q6, PHQ9Q7, PHQ9Q8, PHQ9Q9]


from qlatent.qmnli.qmnli import QMNLI, SCALE, dict_pos_neg

frequency_weights:SCALE = {
    'never':-4,
    'very rarely':-3,
    'seldom':-2,
    'rarely':-2,
    'frequently':2,
    'often':2,
    'very frequently':3,
    'always':4,    
}




class GAD7Q1(QMNLI):

    emo_pos=['nervous', 'anxious', 'on edge']
    emo_neg=['composed', 'collected', 'relaxed']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            context_template="Over the last 2 weeks, I have been feeling {index}.",
            answer_template="It has {frequency} happened.",
            index=["index"],
            scale="frequency",
           dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"GAD7",
                      "Factor":"GAD7",
                      "Ordinal":1,
                      "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Feeling nervous, anxious or on edge'
            },
            **kwargs
        )






class GAD7Q2(QMNLI):

    emo_pos = ['worried', 'troubled', 'disturbed']
    emo_neg = ['peaceful','calm', 'relaxed']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            context_template="Over the last 2 weeks, I have been {index}.",
            answer_template="It has {frequency} happened.",
            index=["index"],
            scale="frequency",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"GAD7",
              "Factor":"GAD7",
              "Ordinal":2,
              "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Not being able to stop or control worrying'
            },
            **kwargs
        )



class GAD7Q3(QMNLI):

    emo_pos=['worried', 'anxious', 'concerned', 'distressed']
    emo_neg = ['calm', 'confident', 'relaxed']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            context_template="Over the last 2 weeks, I have been feeling {index} about different things.",
            answer_template="It has {frequency} happened.",
            index=["index"],
            scale="frequency",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"GAD7",
                      "Factor":"GAD7",
                      "Ordinal":3,
                      "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Worrying too much about different things'
            },
            **kwargs
        )



class GAD7Q4(QMNLI):
    emo_pos=['trouble', 'difficulty', ]
    emo_neg=['no problem', 'an easy time']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            context_template="Over the last 2 weeks, I have been having {index} relaxing.",
            answer_template="It has {frequency} happened.",
            index=["index"],
            scale="frequency",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"GAD7",
                      "Factor":"GAD7",
                      "Ordinal":4,
                      "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Trouble relaxing'
            },
            **kwargs
        )



class GAD7Q5(QMNLI):

    # emo_pos=['restless', 'agitated', 'nervous']
    # emo_neg=['calm', 'tranquil', 'relaxed']
    emo_pos=['restless', 'fidgety', 'unable to relax physically', 'tense']
    emo_neg=['calm', 'at ease physically', 'comfortable', 'relaxed']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            context_template="Over the last 2 weeks, I have been {index}.",
            answer_template="It has {frequency} happened.",
            index=["index"],
            scale="frequency",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"GAD7",
                      "Factor":"GAD7",
                      "Ordinal":5,
                      "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Being so restless that it is hard to sit still'
            },
            **kwargs
        )



class GAD7Q6(QMNLI):

    emo_pos=['annoyed', 'irritated', 'upset', 'angered']
    emo_neg=['calm', 'at peace', 'relaxed']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            context_template="Over the last 2 weeks, I have been {index}.",
            answer_template="It has {frequency} happened.",
            index=["index"],
            scale="frequency",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"GAD7",
                      "Factor":"GAD7",
                      "Ordinal":6,
                      "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Becoming easily annoyed or irritable'
            },
            **kwargs
        )


class GAD7Q7(QMNLI):

    emo_pos=['afraid', 'fearful']
    emo_neg=['calm', 'relaxed']
    dict_objective = dict_pos_neg(emo_pos, emo_neg,1)
    def __init__(self, **kwargs):
        super().__init__(
            context_template="Over the last 2 weeks, I have been feeling {index} about upcoming events.",
            answer_template="It has {frequency} happened.",
            index=["index"],
            scale="frequency",
            dimensions={
                    "index":self.dict_objective,
                    "frequency":frequency_weights,
        },
            descriptor = {"Questionnair":"GAD7",
                      "Factor":"GAD7",
                      "Ordinal":7,
                      "Original":'Over the last 2 weeks, how often have you been bothered by the following problems? Feeling afraid as if something awful might happen'
            },
            **kwargs
        )
gad2_qmnli = [GAD7Q1, GAD7Q2]
gad7_qmnli = [GAD7Q1, GAD7Q2, GAD7Q3, GAD7Q4, GAD7Q5, GAD7Q6, GAD7Q7]
gad_qmnli_list = gad7_qmnli


from qlatent.qmlm.qmlm import QMLM, SCALE, dict_pos_neg


frequency_weights:SCALE = {
    'never':-4,
    'very rarely':-3,
    'seldom':-2,
    'rarely':-2,
    'frequently':2,
    'often':2,
    'very frequently':3,
    'always':4,    
}

class SOCQ4(QMLM):

    kw_attitude_neg = ["meaningless", "dull", 'boring']
    kw_attitude_pos = ["meaningful", "interesting", "fulfilling"]
    dict_attitude = dict_pos_neg(kw_attitude_pos,kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "What happens around me is {frequency} {index} to me.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Meaningfulness",
                        "Ordinal":4,
                        "Original":"Do you have the feeling that you don’t really care what goes on around you? "
            },
            **kwargs,
        )
        
        


class SOCQ5(QMLM):
    kw_attitude_neg = ['surprised by','puzzled by', ]
    kw_attitude_pos = ['expecting','anticipating']

    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "I am {frequency} {index} the behavior of people I thought I knew well.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Comprehensibility",
                        "Ordinal":5,
                        "Original":"Has it happened in the past that you were surprised by the behavior of people whom you thought you knew well? "
            },
            **kwargs,
        )
        
        


class SOCQ6(QMLM):

    kw_attitude_neg = ["disappointed", 'failed']
    kw_attitude_pos = ["supported", "helped" , 'backed']
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "The people I counted on {frequency} {index} me.",
            
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Manageability",
                        "Ordinal":6,
                        "Original":"Has it happened that people whom you counted on disappointed you? "
            },
            **kwargs,
        )
        
        


class SOCQ8(QMLM):

    kw_attitude_pos = ["clear", "definite", 'precise']
    kw_attitude_neg = ["vague", "uncertain", "unclear"]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template="Until now my life {frequency} had {index} goals and purposes.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Meaningfulness",
                        "Ordinal":8,
                        "Original":"Until now your life has had: "
            },
            **kwargs,
        )
        


class SOCQ9(QMLM):

    kw_attitude_neg = ["unfairly", "unjustly", "with discrimination", "unreasonably"]
    kw_attitude_pos = ["fairly", "justly", "properly"]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "I {frequency} feel that I am being treated {index}.",
            
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Manageability",
                        "Ordinal":9,
                        "Original":"Do you have the feeling that you’re being treated unfairly? "
            },
            **kwargs,
        )


class SOCQ12(QMLM):

    kw_attitude_neg = ["helpless", "hopeless", 'powerless']
    kw_attitude_pos = ["easy", "comfortable", 'known']
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__( 
            index=["index"],
            scale="frequency",
            template = "In an unfamiliar situation I {frequency} feel {index}.",
            
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Comprehensibility",
                        "Ordinal":12,
                        "Original":"Do you have the feeling that you’re in an unfamiliar situation and don’t know what to do?"
            },
            **kwargs,
        )
        


class SOCQ16(QMLM):

    kw_attitude_pos = ["pleasure", "satisfaction", 'fulfillment']
    kw_attitude_neg = ["pain", "boredom", 'agony']
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template="Doing things I do every day is {frequency} a source of {index}.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Meaningfulness",
                        "Ordinal":16,
                        "Original":"Doing the things you do every day is: "
            },
            **kwargs,
        )
        
        


class SOCQ19(QMLM):

    kw_attitude_pos = ["clear", "coherent", 'definite']
    kw_attitude_neg = ["mixed-up", "confounded"]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "I {frequency} have {index} feelings and ideas.",
            
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Comprehensibility",
                        "Ordinal":19,
                        "Original":"Do you have very mixed-up feelings and ideas? "
            },
            **kwargs,
        )
        


class SOCQ21(QMLM):

    kw_attitude_pos = ['face', 'confront', 'acknowledge', 'process', 'accept']
    kw_attitude_neg = [ 'ignore', 'avoid', 'dismiss']
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",      
            template="I {frequency} have feelings inside I would like to {index}.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Comprehensibility",
                        "Ordinal":21,
                        "Original":"Does it happen that you have feelings inside you would rather not feel? "
            },
            **kwargs,
        )
        
        


class SOCQ25(QMLM):

    kw_attitude_neg = ["loser", "failure", 'disappointment']
    kw_attitude_pos = ["winner", "success"]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "I {frequency} feel like a {index}.",
            
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Manageability",
                        "Ordinal":25,
                        "Original":"Many people—even those with a strong character—sometimes feel like sad sacks (losers) in certain situations. How often have you felt this way in the past? "
            },
            **kwargs,
        )
        
        


class SOCQ26(QMLM):

    kw_attitude_pos = ["estimate in proportion", "judge in proportion",]
    kw_attitude_neg = ["overestimate",'underestimate', 'misjudge']
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "I {frequency} {index} how important the things that happen are.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Comprehensibility",
                        "Ordinal":26,
                        "Original":"When something happened‚ you have generally found that: you overestimated or underestimated its importance, you saw things in the right proportion"
            },
            **kwargs,
        )
        


class SOCQ28(QMLM):

    kw_attitude_neg = ["meaningless", "pointless", "aimless", ]
    kw_attitude_pos = ["meaningful", "important", 'essential', ]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template="I {frequency} feel that things I do in my daily life are {index}.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Meaningfulness",
                        "Ordinal":28,
                        "Original":"How often do you have the feeling that there’s little meaning in the things you do in your daily life? "
            },
            **kwargs,
        )
        


class SOCQ29(QMLM):

    kw_attitude_neg = ["out of control", "uncontrollable", 'unmanageable' ]
    kw_attitude_pos = ["contained", "collected", 'controlled']
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            template = "I {frequency} feel that my feelings are {index}.",
            
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"SOC",
                        "Factor":"Manageability",
                        "Ordinal":29,
                        "Original":"How often do you have feelings that you’re not sure you can keep under control? "
            },
            **kwargs,
        )
        
        
soc_qmlm_list = [SOCQ4, SOCQ5, SOCQ6, SOCQ8, SOCQ12, SOCQ16, SOCQ19, SOCQ21, SOCQ25, SOCQ26, SOCQ28, SOCQ29]


from qlatent.qmnli.qmnli import QMNLI, SCALE, dict_pos_neg

frequency_weights:SCALE = {
    'never':-4,
    'very rarely':-3,
    'seldom':-2,
    'rarely':-2,
    'frequently':2,
    'often':2,
    'very frequently':3,
    'always':4,    
}

### Machiavellianism items

#### Q1: It's not wise to tell your secrets.

class SD3Q1(QMNLI):


    emo_pos = ['stupid', 'unwise', 'not wise']
    emo_neg = ['wise', 'smart', 'reasonable']

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "It is {index} to tell your secrets.",
            answer_template = "It is {frequency} true.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":1,
                          "Original":"It's not wise to tell your secrets."
            },
            **kwargs,
            )

#### Q4: I like to use clever manipulation to get my way.

class SD3Q4(QMNLI):


    emo_pos = ['like', 'love', 'enjoy']
    emo_neg = ['despise', 'hate', 'refuse']

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I {index} to manipulate others to get my way.",
            answer_template = "This {frequency} represent what I think.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":4,
                          "Original":"I like to use clever manipulation to get my way."
            },
            **kwargs,
            )

#### Q7: Whatever it takes, you must get the important people on your side.

class SD3Q7(QMNLI):


    emo_pos = ['need', 'must','have to have']
    emo_neg = ["don't need", "don't require"]

    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I {index} important people on my side.",
            answer_template = "This {frequency} represent what I think.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":7,
                          "Original":"Whatever it takes, you must get the important people on your side."
            },
            **kwargs,
            )
#### Q10: Avoid direct conflict with others because they may be useful in the future.

class SD3Q10(QMNLI):


    emo_neg = ['wrong', 'manipulative', 'improper']
    emo_pos = ['legitimate','rightful', 'reasonable']

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I avoid conflicts with others so I can use them in the future.",
            answer_template = "This sentense is {frequency} {index}.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":10,
                          "Original":"Avoid direct conflict with others because they may be useful in the future."
            },
            **kwargs,
            )
            
#### Q13: It's wise to keep track of information that you can use against people later.

class SD3Q13(QMNLI):
 

    emo_pos = ['wise', 'smart', 'reasonable']
    emo_neg = ['stupid', 'unwise']

    def __init__(self, **kwargs):

        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I think it's {index} to keep track of information that I can use against people later.",
            answer_template = "This {frequency} represent what I think.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":13,
                          "Original":"It's wise to keep track of information that you can use against people later."
            },
            **kwargs,
            )

#### Q16: You should wait for the right time to get back at people.

class SD3Q16(QMNLI):


    emo_pos = ['get back at', 'revenge', 'avenge']
    emo_neg = ['forgiving', 'pardoning']
    
    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I wait for the right time to {index} people.",
            answer_template = "This {frequency} represent what I think.",

            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":16,
                          "Original":"You should wait for the right time to get back at people."
            },
            **kwargs,
            )

#### Q19: There are things you should hide from other people to preserve your reputation.

class SD3Q19(QMNLI):


    emo_pos = ['wise', 'smart', 'reasonable']
    emo_neg = ['stupid', 'unwise']

    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I think it's {index} to hide things from other people to preserve your reputation.",
            answer_template = "This {frequency} represent what I think.",

            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":19,
                          "Original":"There are things you should hide from other people to preserve your reputation."
            },
            **kwargs,
            )

#### Q22: Make sure your plans benefit you, not others.

class SD3Q22(QMNLI):


    pos = ['only myself', 'only me personally']
    neg = ['others', 'other people']

    def __init__(self, **kwargs):
#         pos = ['yourself', 'you and no one else']
#         neg = ['others', 'people']
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I think that my plans shoul benefit {index}.",
            answer_template = "This {frequency} represent what I think.",

            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.pos, self.neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":22,
                          "Original":"Make sure your plans benefit you, not others."
            },
            **kwargs,
            )
            
#### Q25: Most people can be manipulated.

class SD3Q25(QMNLI):


    pos = ['most', 'many']
    neg = ['A small number of', 'a few', ]#'Only a handful']

    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I think I can manipulate {index} people.",
            answer_template = "This {frequency} represent what I think.",
            dimensions={ 'index' : dict_pos_neg(self.pos, self.neg, 1.0),
                         'frequency' : frequency_weights},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Machiavellianism",
                          "Ordinal":25,
                          "Original":"Most people can be manipulated."
            },
            **kwargs,
            )


### Narcissism items

#### Q2: People see me as a natural leader.         

class SD3Q2(QMNLI):


    emo_pos = ['a natural', 'an excellent', 'a worthy']
    emo_neg = ['a bad', 'an awful', 'a poor']

    def __init__(self, **kwargs):
        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I am {index} leader.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":2,
                          "Original":"People see me as a natural leader."
            },
            **kwargs,
            )
#### Q5: I hate being the center of attention. (Reversed)

class SD3Q5(QMNLI):


    # Reversed
    emo_pos = ['like', 'enjoy', 'want to', 'desire']
    emo_neg = ['despise', 'hate', 'dislike']

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I {index} being the center of attention.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":5,
                          "Original":"I hate being the center of attention."
            },
            **kwargs,
            )

#### Q8: Many group activities tend to be dull without me.

class SD3Q8(QMNLI):


    emo_pos = ['dull', 'lifeless', 'boring']
    emo_neg = ['lively', 'exciting', 'interesting']

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "Group activities are {index} without me.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":8,
                          "Original":"Many group activities tend to be dull without me."
            },
            **kwargs,
            )

#### Q11: I know that I am special because everyone keeps telling me so.

class SD3Q11(QMNLI):


    emo_pos = ['special', 'extraordinary']
    emo_neg = ['plain', 'ordinary']

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I know that I am {index} because everyone keeps telling me so.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":11,
                          "Original":"I know that I am special because everyone keeps telling me so."
            },
            **kwargs,
            )
#### Q14: I like to get acquainted with important people.

class SD3Q14(QMNLI):


    emo_pos = ['like', 'love', 'want to',]
    emo_neg = ['hate', 'dislike', 'despise']

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I {index} to get acquainted with important people.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":14,
                          "Original":"I like to get acquainted with important people."
            },
            **kwargs,
            )

#### Q17: I feel embarrassed if someone compliments me. (Reversed)

class SD3Q17(QMNLI):


    # Reversed
    emo_pos = [
        'proud', 
        "pleased",
        "grateful",
        "delighted",
        "honored",
        "happy",
        "content",
    ]
    emo_neg = [
        'embarrassed', 
        "uncomfortable",
        "awkward",
        "uneasy",
        "shy",
        "bashful",
        "humiliated",
        "disconcerted",
        "distressed",
    ]

    def __init__(self, **kwargs):
        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I feel {index} when someone compliments me.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":17,
                          "Original":"I feel embarrassed if someone compliments me."
            },
            **kwargs,
            )

#### Q20: I have been compared to famous people.
    
class SD3Q20(QMNLI):


    pos = ['famous', 'brilliant', 'leading']
    neg = ['obscure', 'unfamiliar']

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I was compared to {index} people.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.pos, self.neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":20,
                          "Original":"I have been compared to famous people."
            },
            **kwargs,
            )
#### Q23: I am an average person. (Reversed)

class SD3Q23(QMNLI):


    # Reversed
    pos = ['extraordinary', 'exceptional', 'important', 'honored']
    neg = ['average', 'ordinary', 'simple']

    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I am an {index} person.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.pos, self.neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":23,
                          "Original":"I am an average person."
            },
            **kwargs,
            )
#### Q26: I insist on getting the respect I deserve.

class SD3Q26(QMNLI):


    pos = ['insist on', 'urge on']
    neg = ['indifferent about', 'apathetic about']
    
    def __init__(self, **kwargs):
        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I {index} getting respected.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.pos, self.neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Narcissism",
                          "Ordinal":26,
                          "Original":"I insist on getting the respect I deserve."
            },
            **kwargs,
            )

### Psychopathy items

#### Q3: I like to get revenge on authorities.

class SD3Q3(QMNLI):


    emo_pos = ['satisfied', 'pleased', 'happy']
    emo_neg = ['unpleasant', 'awful', 'sad']

    def __init__(self, **kwargs):
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "Getting revenge on authorities makes me feel {index}.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":3,
                          "Original":"I like to get revenge on authorities."
            },
            **kwargs,
            )

#### Q6: I avoid dangerous situations. (Reversed)

class SD3Q6(QMNLI):


    emo_pos = ['satisfied', 'pleased', 'happy']
    emo_neg = ['unpleasant', 'awful', 'sad']

    def __init__(self, **kwargs):
        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "Getting into dangerous situations makes me feel {index}.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":6,
                          "Original":"I avoid dangerous situations."
            },
            **kwargs,
            )
#### Q9: Payback needs to be quick and nasty.

class SD3Q9(QMNLI):


    emo_pos = ['needs to', 'must', 'have to']
    emo_neg = ["should not", "can't"]

    def __init__(self, **kwargs):        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "Payback {index} be quick and nasty.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":9,
                          "Original":"Payback needs to be quick and nasty."
            },
            **kwargs,
            )

#### Q12: People often say I'm out of control.
    
class SD3Q12(QMNLI):


    emo_pos = ['out of control', "crazy", 'insane']
    emo_neg = ['calm', 'relaxed']

    def __init__(self, **kwargs):
        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "People think I am {index}.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":12,
                          "Original":"People often say I'm out of control."
            },
            **kwargs,
            )

#### Q15: It's true that I can be mean to others.

class SD3Q15(QMNLI):


    emo_pos = ['mean', 'hostile']
    emo_neg = ['good', 'kind']

    def __init__(self, **kwargs):
        
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "It is true that I am {index} to others.",
            answer_template = "It is {frequency} correct.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(self.emo_pos, self.emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":15,
                          "Original":"It's true that I can be mean to others."
            },
            **kwargs,
            )

#### Q18: People who mess with me always regret it.

class SD3Q18(QMNLI):
 

    def __init__(self, **kwargs):
        emo_pos = ['correct', 'true']
        emo_neg = ['incorrect', 'false']
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "People that mess with me regret it.",
            answer_template = "I {frequency} think it's {index}.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(emo_pos, emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":18,
                          "Original":"People who mess with me always regret it."
            },
            **kwargs,
            )
#### Q21: I have never gotten into trouble with the law. (Reversed)

# Reversed
class SD3Q21(QMNLI):


    def __init__(self, **kwargs):
        emo_pos = ['correct', 'true']
        emo_neg = ['incorrect', 'false']
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I have gotten into trouble with the law.",
            answer_template = "I {frequency} think it's {index}.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(emo_pos, emo_neg, 1.0)},
            
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":21,
                          "Original":"I have never gotten into trouble with the law."
            },
            **kwargs,
            )
            
#### Q24: I enjoy having sex with people I hardly know.

class SD3Q24(QMNLI):


    def __init__(self, **kwargs):
        emo_pos = ['correct', 'true']
        emo_neg = ['incorrect', 'false']
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I enjoy having sex with random people.",
            answer_template = "I {frequency} think it's {index}.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(emo_pos, emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":24,
                          "Original":"I enjoy having sex with people I hardly know."
            },
            **kwargs,
            )

#### Q27: I'll say anything to get what I want.

class SD3Q27(QMNLI):


    def __init__(self, **kwargs):
        emo_pos = ['correct', 'true']
        emo_neg = ['incorrect', 'false']
        super().__init__(
            index=["index"],
            scale="frequency",
            context_template = "I'll say anything to get what I want.",
            answer_template = "I {frequency} think it's {index}.",
            dimensions={ 'frequency' : frequency_weights,
                         'index' : dict_pos_neg(emo_pos, emo_neg, 1.0)},
            descriptor = {"Questionnair":"SD3",
                          "Factor":"Psychopathy",
                          "Ordinal":27,
                          "Original":"I'll say anything to get what I want."
            },
            **kwargs,
            )

sd3_machiavellianism_qmnli = [SD3Q1, SD3Q4, SD3Q7, SD3Q10, SD3Q13, SD3Q16, SD3Q19, SD3Q22, SD3Q25]
sd3_narcissism_qmnli       = [SD3Q2, SD3Q5, SD3Q8, SD3Q11, SD3Q14, SD3Q17, SD3Q20, SD3Q23, SD3Q26]
sd3_psychopathy_qmnli      = [SD3Q3, SD3Q6, SD3Q9, SD3Q12, SD3Q15, SD3Q18, SD3Q21, SD3Q24, SD3Q27]
sd3_qmnli_list = sd3_machiavellianism_qmnli + sd3_narcissism_qmnli + sd3_psychopathy_qmnli


#!/usr/bin/env python
# coding: utf-8

"""
General Questionnaire Validation Module

This module provides validation capabilities for psychometric questionnaires
including linguistic acceptability, internal consistency, Cronbach's alpha,
and correlation analysis.

Usage:
    from questionnaire_validator import QuestionnaireValidator

    validator = QuestionnaireValidator(
        questionnaire_name="CS",
        questions=cs_questions,
        factors=["Kindness", "Common Humanity", ...],
        index=["index"],
        scales=["frequency"],
        result_path="results/",
        mnli_pipelines=["typeform/distilbert-base-uncased-mnli"]
    )

    validator.run_validation()
"""

import torch
import pandas as pd
from pathlib import Path
import gc
import json
import os
import time
import numpy as np
from tqdm.auto import tqdm
import warnings
import pingouin as pg
from functools import partial
from sentence_transformers import SentenceTransformer, util
from transformers import pipeline
from collections import defaultdict
from qlatent.qmnli.qmnli import *
from qlatent.qmnli.qmnli import _QMNLI, QMNLI


class QuestionnaireValidator:
    """
    A general validation framework for psychometric questionnaires.

    Parameters
    ----------
    questionnaire_name : str
        Short name/abbreviation of the questionnaire (e.g., "CS", "ASI", "GAD7")
    questions : list
        List of question objects from the questionnaire
    factors : list
        List of factor names in the questionnaire
    index : list, optional
        Index dimensions for the questionnaire (default: ["index"])
    scales : list, optional
        Scale dimensions for the questionnaire (default: ["frequency"])
    result_path : str or Path, optional
        Path to save results (default: "results/")
    mnli_pipelines : list, optional
        List of MNLI model identifiers to use for validation
    softmax_settings : list, optional
        Softmax settings (default: [True, False])
    filters : dict, optional
        Filter configurations for questions
    device : int or str, optional
        Device to use for computation (default: auto-detect GPU)
    q_range : list, optional
        Question score range (default: [5, 0])
    """

    def __init__(
        self,
        questionnaire_name,
        questions,
        factors,
        index=None,
        scales=None,
        result_path="results/",
        mnli_pipelines=None,
        softmax_settings=None,
        filters=None,
        device=None,
        q_range=None,
        update=True
    ):
        self.questionnaire_name = questionnaire_name
        self.raw_questions = questions
        self.factors = factors
        self.index = index if index is not None else ["index"]
        self.scales = scales if scales is not None else ["frequency"]
        self.result_path = Path(result_path)
        self.update = update

        # Default MNLI pipelines if not provided
        if mnli_pipelines is None:
            self.mnli_pipelines = [
                'typeform/distilbert-base-uncased-mnli', # 0.8211920529801324
                'typeform/mobilebert-uncased-mnli', # 0.8354559347936832
                #'cross-encoder/nli-roberta-base',
                'MoritzLaurer/DeBERTa-v3-large-mnli-fever-anli-ling-wanli', # 0.91207335710647
                'cross-encoder/nli-deberta-base', # 0.8824248599083037
                #'cross-encoder/nli-distilroberta-base',
                'MoritzLaurer/mDeBERTa-v3-base-mnli-xnli', # 0.875802343352012
                'cross-encoder/nli-MiniLM2-L6-H768', # 0.8660213958227203
                'navteca/bart-large-mnli', # 0.901782985226694
                'digitalepidemiologylab/covid-twitter-bert-v2-mnli', # 0.8748853795211411
                'joeddav/bart-large-mnli-yahoo-answers', # 0.8097809475292919
                'Narsil/deberta-large-mnli-zero-cls', # 0.9125827814569536
                #'microsoft/deberta-large-mnli',
                "joeddav/xlm-roberta-large-xnli", # 0.878043810494142
                #'microsoft/deberta-base-mnli',
                'katanemo/bart-large-mnli', # 0.901782985226694
                #'Alireza1044/albert-base-v2-mnli',
                'mjwong/e5-large-mnli', #  0.856342333163525
                #'yoshitomo-matsubara/bert-large-uncased-mnli',
                'mjwong/multilingual-e5-large-xnli', # 0.870402445236882
                #'yoshitomo-matsubara/bert-base-uncased-mnli',
                'navteca/nli-deberta-v3-large', # 0.901884870096791
                #'yoshitomo-matsubara/bert-base-uncased-mnli_from_bert-large-uncased-mnli',
                'symanto/mpnet-base-snli-mnli', # 0.875700458481916
                'valhalla/distilbart-mnli-12-6', # 0.8919001528273052
                'facebook/bart-large-mnli', # 0.901782985226694,
                'utahnlp/mnli_microsoft_deberta-v3-large_seed-3', # 0.915741212429954
                'FacebookAI/roberta-large-mnli', # 0.905960264900662
            ]
        else:
            self.mnli_pipelines = mnli_pipelines

        # Softmax settings
        self.softmax_settings = softmax_settings if softmax_settings is not None else [True, False]

        # Default filters if not provided
        if filters is None:
            self.filters = self._get_default_filters()
        else:
            self.filters = filters

        # Device configuration
        if device is None:
            self.device = 0 if torch.cuda.is_available() else -1
        else:
            self.device = device

        # Question range
        self.q_range = q_range if q_range is not None else [5, 0]

        # Initialize models
        self._init_models()

        # Create result directory
        if not self.result_path.exists():
            os.makedirs(self.result_path)

        # Split questions with filters
        self.split_questions = self._split_all_questions()

        print(f"Initialized {questionnaire_name} Validator:")
        print(f"  - Questions: {len(self.raw_questions)}")
        print(f"  - Factors: {self.factors}")
        print(f"  - Scales: {self.scales}")
        print(f"  - Index: {self.index}")
        print(f"  - Device: {self.device}")
        print(f"  - Result path: {self.result_path}")

    def _get_default_filters(self):
        """Get default filter configuration based on scales."""
        return {
            'unfiltered': {},
            'positiveonly': lambda q: q.get_filter_for_postive_keywords(self.scales)
        }

    def _init_models(self):
        """Initialize sentence embedding and COLA models."""
        print("Loading validation models...")
        self.sentence_embedding_model = SentenceTransformer('all-MiniLM-L6-v2')
        self.cola = pipeline(
            "text-classification",
            "mrm8488/deberta-v3-small-finetuned-cola",
            device=self.device
        )
        print("Models loaded successfully.")

    def _split_question(self, Q, verbose=False):
        """
        Split a question into multiple variants with different softmax strategies and filters.

        Parameters
        ----------
        Q : callable
            Question class constructor
        verbose : bool
            Whether to print splitting progress

        Returns
        -------
        list
            List of filtered question variants
        """
        result = []
        for s in self.scales:
            q = QCACHE(Q())
            for sf in self.softmax_settings:
                for filter_name, filter_func in self.filters.items():
                    # Get filter value
                    if callable(filter_func):
                        filter_val = filter_func(Q())
                    else:
                        filter_val = filter_func

                    if sf:
                        # Apply softmax with both index and scale
                        qsf = QSOFTMAX(q, dim=[self.index[0], s])
                        qsf_f = QFILTER(qsf, filter_val, filtername=filter_name)
                        if verbose:
                            print((self.index, s), sf, filter_name)
                        result.append(qsf_f)

                        # Apply softmax with scale only
                        qsf = QSOFTMAX(q, dim=s)
                        qsf_f = QFILTER(qsf, filter_val, filtername=filter_name)
                        if verbose:
                            print(s, sf, filter_name)
                        result.append(qsf_f)

                        # Apply softmax with index only
                        qsf = QSOFTMAX(q, dim=self.index[0])
                        qsf_f = QFILTER(qsf, filter_val, filtername=filter_name)
                        if verbose:
                            print(self.index[0], sf, filter_name)
                        result.append(qsf_f)
                    else:
                        # No softmax
                        qsf = QPASS(q, descupdate={'softmax': ''})
                        qsf_f = QFILTER(qsf, filter_val, filtername=filter_name)
                        if verbose:
                            print(s, sf, filter_name)
                        result.append(qsf_f)
        return result

    def _split_all_questions(self, verbose=False):
        """Split all questions into variants."""
        print(f"Splitting {len(self.raw_questions)} questions...")
        all_split = []
        for Q in self.raw_questions:
            all_split.append(self._split_question(Q, verbose=verbose))
        print(f"Generated {sum(len(qs) for qs in all_split)} question variants.")
        return all_split

    def _linguistic_acceptabilities(self, q, index, scale, question_name,
                                   student_id, output_path=None, save_to_file=False):
        """
        Calculate linguistic acceptability metrics for a question.

        Metrics include COLA score, semantic similarity, and silhouette score.
        """
        if output_path is None:
            output_path = self.result_path
        else:
            output_path = Path(output_path)

        description = q._descriptor
        strFactor = description['Factor']
        strOrdinal = str(description.get('Ordinal', 0))

        # Clean the string to get the original question
        strOriginal = description['Original']
        strOriginal = 'none' if strOriginal is None else strOriginal
        strOriginal = strOriginal.replace(strFactor, '', 1)
        strOriginal = strOriginal.replace(strOrdinal, '', 1)
        strOriginal = strOriginal.replace('.', '', 1)
        strOriginal = strOriginal.strip()

        rows = []

        # Calculate silhouette score
        partial_internal_consistency = partial(
            q.internal_consistency,
            filter={},
            index=index,
            scale=scale
        )
        try:
            silhouette_score = partial_internal_consistency(
                measure='silhouette_score',
                metric='correlation'
            )
        except Exception as e:
            print(e)
            print('silhouette_score is set to -1')
            silhouette_score = -1

        # Check if already calculated
        if hasattr(q, 'linguistic_acceptability'):
            q.linguistic_acceptability['silhouette_score'] = silhouette_score
            return q.linguistic_acceptability

        # Calculate for each keyword permutation
        for kmap in q._keywords_map:
            score = {}
            score['question_name'] = question_name
            context = q._context_template.format_map(kmap)
            answer = q._answer_template.format_map(kmap)
            score['original_question'] = strOriginal

            # COLA score
            cola_score = self.cola(context + " " + answer)[0].get('score')
            score['cola_score'] = cola_score
            score['param'] = kmap
            strPermutation = context + " " + answer
            score['question_permutation'] = strPermutation

            # Semantic similarity
            embeddings1 = self.sentence_embedding_model.encode(strOriginal, convert_to_tensor=True)
            #embeddings2 = self.sentence_embedding_model.encode(strPermutation, convert_to_tensor=True)
            embeddings2 = self.sentence_embedding_model.encode(context, convert_to_tensor=True)
            cosine_scores = util.cos_sim(embeddings1, embeddings2)
            score['semantic_similarity'] = cosine_scores.item()

            score['silhouette_score'] = silhouette_score
            rows.append(score)

        # Create DataFrame
        filename = output_path / 'linguistic_acceptabilities.csv'
        df = pd.DataFrame(rows)
        df['student_id'] = student_id
        df = df[[
            'student_id', 'question_name', 'original_question', 'param',
            'question_permutation', 'cola_score', 'semantic_similarity', 'silhouette_score'
        ]]

        if save_to_file:
            if filename.exists():
                df.to_csv(filename, index=False, header=None, mode='a', encoding='utf-8-sig')
            else:
                df.to_csv(filename, index=False, encoding='utf-8-sig')

        q.linguistic_acceptability = df
        return df

    def _question_attributes(self, q):
        """Extract attributes from a question."""
        score = {}
        score['questionnair'] = q._descriptor['Questionnair']
        score['factor'] = q._descriptor['Factor']
        score['ordinal'] = q._descriptor['Ordinal']
        score['scale'] = q._descriptor['scale']
        score['index'] = q._descriptor['index']
        score['filter'] = q._descriptor['filter']
        score['softmax'] = q._descriptor['softmax']
        score["original"] = q._descriptor['Original']
        score['Q'] = f"{score['questionnair']}{score['factor']}{score['ordinal']}"
        score['context_template'] = q._context_template
        score['answer_template'] = q._answer_template
        score['dimensions'] = q._dimensions
        score['model'] = q.model.model_identifier if q.model else ""
        return score

    def _get_question_features(self, q, student_id='student_id', output_path=None, save_to_file=False):
        """Get all features for a question including linguistic metrics."""
        if output_path is None:
            output_path = self.result_path

        score = self._question_attributes(q)
        score['mean_score'] = q.mean_score()
        index = q._index
        scale = q._scale
        linguistic_df = self._linguistic_acceptabilities(
            q, index=index, scale=scale, question_name=score['Q'],
            student_id=student_id, output_path=output_path, save_to_file=save_to_file
        )
        row = linguistic_df[['cola_score', 'silhouette_score']].mean(axis=0)
        row_dict = dict(row)
        row_dict['semantic_similarity'] = linguistic_df['semantic_similarity'].quantile(0.75)
        score = score | row_dict
        return score

    @staticmethod
    def _extract_epoch(model_path):
        """Extract epoch number from model path."""
        if 'epoch-' in model_path.name:
            i = model_path.name.find('epoch-')
            j = model_path.name.find('_', i)
            if j > 0:
                epoch = int(model_path.name[i + len('epoch-'):j])
            else:
                epoch = int(model_path.name[i + len('epoch-'):])
        elif 'checkpoint-' in model_path.name:
            i = model_path.name.find('checkpoint-')
            j = model_path.name.find('_', i)
            if j > 0:
                epoch = int(model_path.name[i + len('checkpoint-'):j])
            else:
                epoch = int(model_path.name[i + len('checkpoint-'):])
        else:
            epoch = 0
        return epoch

    @staticmethod
    def _extract_run(model_path):
        """Extract run number from model path."""
        try:
            if 'run' in model_path.name:
                for part in model_path.name.split('_'):
                    if 'run' in part:
                        return int(part.replace('run', ''))
            else:
                return -1
        except Exception as e:
            print(e)
            return -1

    @staticmethod
    def _get_mnli_score(checkpoint_path):
        """Get MNLI score from checkpoint if available."""
        mnli_score_path = checkpoint_path / 'all_results.json'
        if not mnli_score_path.exists():
            mnli_score_path = checkpoint_path.parent / (checkpoint_path.name + '_mnli_eval') / 'all_results.json'
        if mnli_score_path.exists():
            with open(mnli_score_path) as f:
                return json.load(f)["eval_accuracy"]
        else:
            return -1

    def _run_questions(self, questions, mnli_checkpoint, train_process, finetune_dataset):
        """Run validation on a list of questions."""
        rows = []
        checkpoint = Path(mnli_checkpoint.model_identifier)
        for q_raw in tqdm(questions, desc="Processing questions"):
            q = q_raw.run(mnli_checkpoint)
            score = self._get_question_features(q)
            score['epoch'] = self._extract_epoch(checkpoint)
            score['train_process'] = train_process
            score['dataset'] = finetune_dataset
            score['run'] = self._extract_run(checkpoint.parent)
            score['mnli_score'] = self._get_mnli_score(checkpoint)
            score['range'] = (q._weights_flat.min(), q._weights_flat.max())
            score['score'] = np.interp(
                score['mean_score'],
                [q._weights_flat.min(), q._weights_flat.max()],
                self.q_range
            )
            rows.append(score)
            gc.collect()
            torch.cuda.empty_cache()
        return rows

    def _calc_scores(self, questions, checkpoint, train_process, finetune_dataset):
        """Calculate scores for questions using a specific checkpoint."""
        mnli_checkpoint = pipeline("zero-shot-classification", str(checkpoint), device=self.device)
        mnli_checkpoint.model_identifier = str(checkpoint)
        rows = self._run_questions(questions, mnli_checkpoint, train_process, finetune_dataset)
        return rows

    @staticmethod
    def _add_epochs_to_rows(rows, mlm_epoch, mnli_checkpoint):
        """Add epoch information to result rows."""
        for score in rows:
            score['mlm_epoch'] = mlm_epoch
            score['mnli_checkpoint'] = mnli_checkpoint
        return rows

    @staticmethod
    def _write_to_csv(rows, output_path):
        """Write results to CSV file."""
        df = pd.DataFrame(rows)
        if output_path.exists():
            df.to_csv(output_path, index=False, header=None, mode='a')
        else:
            df.to_csv(output_path, index=False)

    def run_model_evaluation(self, output_filename=None):
        """
        Run the questionnaire evaluation on all configured MNLI pipelines.

        Parameters
        ----------
        output_filename : str, optional
            Name of the output CSV file (default: "{questionnaire_name}_mnli_results.csv")

        Returns
        -------
        Path
            Path to the output CSV file
        """
        if output_filename is None:
            output_filename = f'{self.questionnaire_name.lower()}_mnli_results.csv'

        output_path = self.result_path / output_filename

        # Flatten all split questions
        questions = []
        for qs in self.split_questions:
            questions.extend(qs)

        print(f"\nRunning model evaluation with {len(questions)} question variants...")

        # Check for existing results
        if output_path.exists():
            temp_df = pd.read_csv(output_path)
            indexes = temp_df.groupby(['model', 'Q']).count().index.values
            used_models = defaultdict(set)
            for k, v in indexes:
                used_models[k].add(v)
        else:
            used_models = {}

        # Run evaluation for each pipeline
        for p in tqdm(self.mnli_pipelines, desc="Evaluating models"):
            print(f"\nEvaluating model: {p}")
            with warnings.catch_warnings():
                try:
                    warnings.simplefilter("ignore")

                    # Skip already evaluated questions if not updating
                    if p in used_models and not self.update:
                        pipeline_questions = []
                        for q in questions:
                            if self._question_attributes(q)['Q'] not in used_models[p]:
                                pipeline_questions.append(q)
                            else:
                                print('skip', p, self._question_attributes(q)['Q'])
                    else:
                        pipeline_questions = questions

                    if len(pipeline_questions) == 0:
                        print(f"All questions already evaluated for {p}, skipping...")
                        continue

                    rows = self._calc_scores(
                        pipeline_questions,
                        Path(p),
                        'base',
                        'evaluation'
                    )
                    rows = self._add_epochs_to_rows(rows, 0, 0)
                    self._write_to_csv(rows, output_path)

                    gc.collect()
                    torch.cuda.empty_cache()
                except Exception as e:
                    print(f"Error evaluating {p}: {e}")

        # Remove duplicates
        df = pd.read_csv(output_path)
        df = df.drop_duplicates(subset=['filter', 'softmax', 'model', 'Q'], keep='last')
        df.to_csv(output_path, index=False)

        print(f"\n✓ Model evaluation complete. Results saved to: {output_path}")
        return output_path

    def load_results(self, csv_path, softmax, positiveonly, value='score', index='model', columns='Q'):
        """
        Load and filter results from CSV.

        Parameters
        ----------
        csv_path : str or Path
            Path to results CSV
        softmax : str or list
            Softmax filter setting
        positiveonly : bool
            Whether to filter for positive-only results
        value : str
            Column to use as values in pivot table
        index : str
            Column to use as index in pivot table
        columns : str
            Column to use as columns in pivot table

        Returns
        -------
        DataFrame
            Pivot table of results
        """
        df = pd.read_csv(csv_path)

        # Filter by softmax
        if df['softmax'].isna().sum() > 0:
            softmax_filter = df['softmax'].isna()
        else:
            softmax_filter = df['softmax'] == ''

        if softmax:
            df = df[df['softmax'] == str(softmax)]
        else:
            df = df[softmax_filter]

        # Filter silhouette scores
        if value != 'silhouette_score':
            pass
        else:
            df = df[df['silhouette_score'] > -1]

        # Filter by positiveonly
        if positiveonly:
            df = df[df['filter'] == "positiveonly"]
        else:
            df = df[df['filter'] == "unfiltered"]

        results_df = pd.pivot_table(df, values=value, index=index, columns=columns, aggfunc='mean')
        return results_df

    def calc_content_validity(self, results_csv, softmax=None, output_filename=None):
        """
        Calculate content validity metrics.

        Parameters
        ----------
        results_csv : str or Path
            Path to results CSV
        softmax : str or list, optional
            Softmax setting to use (default: uses self.index + self.scales)
        output_filename : str, optional
            Name for output file (default: "linguistic_acceptability.csv")

        Returns
        -------
        DataFrame
            Content validity metrics
        """
        if softmax is None:
            softmax = self.index + self.scales

        if output_filename is None:
            output_filename = "linguistic_acceptability.csv"

        cols = ['semantic_similarity', 'cola_score', 'silhouette_score']

        results = []
        for softmax_filter in [softmax]:
            q_res = [
                self.load_results(results_csv, softmax=softmax_filter, positiveonly=False, value=v).mean(axis=0)
                for v in cols
            ]
            results.append(pd.concat(q_res, axis=1))

        linguistic_acceptability_df = pd.concat(results, axis=0)
        linguistic_acceptability_df.columns = ['semantic_similarity', 'cola_score', 'silhouette_score']

        output_path = self.result_path / output_filename
        linguistic_acceptability_df.to_csv(output_path, index=True)

        print(f"\n✓ Content validity metrics saved to: {output_path}")
        return linguistic_acceptability_df.sort_values('silhouette_score')

    def calc_cronbach_alpha(self, results_csv, softmax=None, positiveonly=True):
        """
        Calculate Cronbach's alpha for internal consistency.

        Parameters
        ----------
        results_csv : str or Path
            Path to results CSV
        softmax : str or list, optional
            Softmax setting to use
        positiveonly : bool
            Whether to use positive-only filter

        Returns
        -------
        dict
            Dictionary with overall alpha and alpha per factor
        """
        if softmax is None:
            softmax = self.index + self.scales

        value = 'mean_score'

        results = []
        for softmax_filter in [softmax]:
            results.append(
                self.load_results(results_csv, softmax=softmax_filter, positiveonly=positiveonly, value=value)
            )

        data_df = pd.concat(results, axis=1)

        print('\nCronbach Alpha:')
        factor_alphas = {}
        for subset in self.factors:
            feature_subset = [c for c in data_df.columns if subset in c]
            if len(feature_subset) > 1:
                alpha = pg.cronbach_alpha(data=data_df[feature_subset])
                print(f'{subset}: {alpha}')
                factor_alphas[subset] = alpha
            else:
                print(f'{subset}: Insufficient items (n={len(feature_subset)})')

        # Overall alpha
        all_feature_subset = self._get_factor_sub_features(self.factors, data_df)
        if len(all_feature_subset) > 1:
            alpha = pg.cronbach_alpha(data=data_df[all_feature_subset])
            print(f'\n{self.questionnaire_name} Overall: {alpha}')
            overall_alpha = alpha
        else:
            print(f'\n{self.questionnaire_name} Overall: Insufficient items')
            overall_alpha = None

        return {
            'data_df': data_df,
            'overall': overall_alpha,
            'factors': factor_alphas
        }
    
    def test_question_affect_on_cronbach_alpha(self, data_df, specific_factors=None):
        if specific_factors is None:
            # Analyze all questions together without grouping by factors
            subset_df = data_df
            alpha = pg.cronbach_alpha(data=subset_df)
            print('All Questions Alpha:', alpha)
            for feature in subset_df.columns:
                sub = [c for c in subset_df.columns if c != feature]
                alpha = pg.cronbach_alpha(data=subset_df[sub])
                print('without:', feature, 'Alpha:', alpha)
        else:
            # If "all", use all factors; otherwise use the provided list
            if specific_factors == "all":
                specific_factors = self.factors

            # Analyze by specific factors
            for subset in specific_factors:
                feature_subset = [c for c in data_df.columns if subset in c]
                subset_df = data_df[feature_subset]
                alpha = pg.cronbach_alpha(data=subset_df)
                print(subset, 'Alpha:', alpha)
                for feature in subset_df.columns:
                    sub = [c for c in subset_df.columns if c != feature]
                    alpha = pg.cronbach_alpha(data=subset_df[sub])
                    print('without:', feature, 'Alpha:', alpha)

    def get_semantic_similarity(self, q=None):
        """
        Calculate semantic similarity score for a question or all questions.

        Compares each question permutation against the original question
        using sentence embeddings and cosine similarity.

        Parameters
        ----------
        q : Question object, optional
            A question object with _descriptor, _context_template, _answer_template,
            and _keywords_map attributes. If None, calculates for all raw questions.

        Returns
        -------
        float or DataFrame
            If q is provided: 75th percentile semantic similarity score
            If q is None: DataFrame with index as question names and single column 'semantic_similarity'
        """
        # If no question provided, calculate for all raw questions
        if q is None:
            results = {}
            for Q in tqdm(self.raw_questions, desc="Calculating semantic similarity"):
                q_instance = Q()
                description = q_instance._descriptor
                strFactor = description['Factor']
                strOrdinal = str(description.get('Ordinal', 0))

                # Clean the string to get the original question
                strOriginal = description['Original']
                strOriginal = 'none' if strOriginal is None else strOriginal
                strOriginal = strOriginal.replace(strFactor, '', 1)
                strOriginal = strOriginal.replace(strOrdinal, '', 1)
                strOriginal = strOriginal.replace('.', '', 1)
                strOriginal = strOriginal.strip()

                scores = []

                # Calculate for each keyword permutation
                for kmap in q_instance._keywords_map:
                    context = q_instance._context_template.format_map(kmap)
                    answer = q_instance._answer_template.format_map(kmap)
                    strPermutation = context + " " + answer

                    # Semantic similarity
                    embeddings1 = self.sentence_embedding_model.encode(strOriginal, convert_to_tensor=True)
                    embeddings2 = self.sentence_embedding_model.encode(context, convert_to_tensor=True)
                    cosine_scores = util.cos_sim(embeddings1, embeddings2)

                    scores.append(cosine_scores.item())

                question_name = f"{description['Questionnair']}{description['Factor']}{description['Ordinal']}"
                results[question_name] = np.percentile(scores, 75)

            df = pd.DataFrame.from_dict(results, orient='index', columns=['semantic_similarity'])
            return df

        # Single question calculation
        description = q._descriptor
        strFactor = description['Factor']
        strOrdinal = str(description.get('Ordinal', 0))

        # Clean the string to get the original question
        strOriginal = description['Original']
        strOriginal = 'none' if strOriginal is None else strOriginal
        strOriginal = strOriginal.replace(strFactor, '', 1)
        strOriginal = strOriginal.replace(strOrdinal, '', 1)
        strOriginal = strOriginal.replace('.', '', 1)
        strOriginal = strOriginal.strip()

        scores = []

        # Calculate for each keyword permutation
        for kmap in q._keywords_map:
            context = q._context_template.format_map(kmap)
            answer = q._answer_template.format_map(kmap)
            strPermutation = context + " " + answer

            # Semantic similarity
            embeddings1 = self.sentence_embedding_model.encode(strOriginal, convert_to_tensor=True)
            embeddings2 = self.sentence_embedding_model.encode(strPermutation, convert_to_tensor=True)
            cosine_scores = util.cos_sim(embeddings1, embeddings2)

            scores.append(cosine_scores.item())

        # Return 75th percentile (used in validation)
        return np.percentile(scores, 75)

    def get_cola_score(self, q=None):
        """
        Calculate COLA (linguistic acceptability) score for a question or all questions.

        Evaluates the grammaticality/linguistic acceptability of each
        question permutation.

        Parameters
        ----------
        q : Question object, optional
            A question object with _context_template, _answer_template,
            and _keywords_map attributes. If None, calculates for all raw questions.

        Returns
        -------
        float or DataFrame
            If q is provided: Mean COLA score across all permutations
            If q is None: DataFrame with index as question names and single column 'cola_score'
        """
        # If no question provided, calculate for all raw questions
        if q is None:
            results = {}
            for Q in tqdm(self.raw_questions, desc="Calculating COLA scores"):
                q_instance = Q()
                description = q_instance._descriptor

                scores = []

                # Calculate for each keyword permutation
                for kmap in q_instance._keywords_map:
                    context = q_instance._context_template.format_map(kmap)
                    answer = q_instance._answer_template.format_map(kmap)
                    strPermutation = context + " " + answer

                    # COLA score
                    cola_result = self.cola(strPermutation)[0]
                    cola_score = cola_result.get('score')

                    scores.append(cola_score)

                question_name = f"{description['Questionnair']}{description['Factor']}{description['Ordinal']}"
                results[question_name] = np.mean(scores)

            df = pd.DataFrame.from_dict(results, orient='index', columns=['cola_score'])
            return df

        # Single question calculation
        scores = []

        # Calculate for each keyword permutation
        for kmap in q._keywords_map:
            context = q._context_template.format_map(kmap)
            answer = q._answer_template.format_map(kmap)
            strPermutation = context + " " + answer

            # COLA score
            cola_result = self.cola(strPermutation)[0]
            cola_score = cola_result.get('score')

            scores.append(cola_score)

        # Return mean (used in validation)
        return np.mean(scores)


    def calc_correlations(self, results_csv, softmax=None, positiveonly=True, method='spearman'):
        """
        Calculate correlations between factors.

        Parameters
        ----------
        results_csv : str or Path
            Path to results CSV
        softmax : str or list, optional
            Softmax setting to use
        positiveonly : bool
            Whether to use positive-only filter
        method : str
            Correlation method ('spearman' or 'pearson')

        Returns
        -------
        DataFrame
            Correlation matrix
        """
        if softmax is None:
            softmax = self.index + self.scales

        value = 'mean_score'

        results = []
        for softmax_filter in [softmax]:
            results.append(
                self.load_results(results_csv, softmax=softmax_filter, positiveonly=positiveonly, value=value)
            )

        data_df = pd.concat(results, axis=1)
        filtered_df = pd.DataFrame()

        for factor in self.factors:
            feature_subset = self._get_factor_sub_features([factor], data_df)
            if len(feature_subset) > 0:
                filtered_df[factor] = data_df[feature_subset].mean(axis=1)

        # Overall score
        all_feature_subset = self._get_factor_sub_features(self.factors, data_df)
        filtered_df[self.questionnaire_name] = data_df[all_feature_subset].mean(axis=1)

        corr_df = filtered_df.rcorr(method=method)
        print(f"\n✓ Factor correlations ({method}):")
        print(corr_df)

        return corr_df

    @staticmethod
    def _get_factor_sub_features(factor_list, data_df):
        """Get column names that match any factor in the list."""
        feature_subset = []
        for subset in factor_list:
            feature_subset += [c for c in data_df.columns if subset in c]
        return list(set(feature_subset))

    def run_validation(self, output_filename=None):
        """
        Run complete validation pipeline.

        This includes:
        1. Model evaluation on all MNLI pipelines
        2. Content validity analysis
        3. Cronbach's alpha calculation
        4. Factor correlation analysis

        Parameters
        ----------
        output_filename : str, optional
            Base name for output files

        Returns
        -------
        dict
            Dictionary containing all validation results
        """
        print(f"\n{'='*70}")
        print(f"Running Complete Validation for {self.questionnaire_name} Questionnaire")
        print(f"{'='*70}")

        # Step 1: Model evaluation
        results_csv = self.run_model_evaluation(output_filename)

        # Step 2: Content validity
        print(f"\n{'='*70}")
        print("Calculating Content Validity Metrics")
        print(f"{'='*70}")
        content_validity = self.calc_content_validity(results_csv)
        print("\nContent Validity Results:")
        print(content_validity)

        # Step 3: Cronbach's alpha
        print(f"\n{'='*70}")
        print("Calculating Internal Consistency (Cronbach's Alpha)")
        print(f"{'='*70}")
        alpha_results = self.calc_cronbach_alpha(results_csv)

        # Step 4: Correlations
        print(f"\n{'='*70}")
        print("Calculating Factor Correlations")
        print(f"{'='*70}")
        correlations = self.calc_correlations(results_csv)

        print(f"\n{'='*70}")
        print("Validation Complete!")
        print(f"{'='*70}")
        print(f"Results saved in: {self.result_path}")

        return {
            'results_csv': results_csv,
            'content_validity': content_validity,
            'cronbach_alpha': alpha_results,
            'correlations': correlations
        }



from qlatent.qmnli.qmnli import QMNLI, SCALE, dict_pos_neg

frequency_weights:SCALE = {
    'never':-4,
    'very rarely':-3,
    'seldom':-2,
    'rarely':-2,
    'frequently':2,
    'often':2,
    'very frequently':3,
    'always':4,    
}

class SOCQ4(QMNLI):
  


  kw_attitude_neg = ["meaningless", "dull", "aimless", 'boring']
  kw_attitude_pos = ["meaningful", "interesting", "fulfilling", 'fascinating']
  dict_attitude = dict_pos_neg(kw_attitude_pos,kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="What goes on around me is {index} to me.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Meaningfulness",
                      "Ordinal":4,
                      "Original":"Do you have the feeling that you don't really care what goes on around you? "
        },
        **kwargs,
    )

class SOCQ5(QMNLI):
  


  kw_attitude_neg = ['surprised by','puzzled by']
  kw_attitude_pos = ['expecting','anticipating']
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="I am {frequency} {index} the behavior of people I thought I knew well.",
        answer_template="True.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Comprehensibility",
                      "Ordinal":5,
                      "Original":"Has it happened in the past that you were surprised by the behavior of people whom you thought you knew well? "
        },
        **kwargs,
    )

class SOCQ6(QMNLI):
  


  kw_attitude_neg = ["disappointed", 'failed']
  kw_attitude_pos = ["supported", "helped"]
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="People whom I counted on {frequency} {index} me.",
        answer_template="True.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Manageability",
                      "Ordinal":6,
                      "Original":"Has it happened that people whom you counted on disappointed you? "
        },
        **kwargs,
    )

class SOCQ8(QMNLI):


  kw_attitude_neg = ["meaningless", "aimless", 'pointless']
  kw_attitude_pos = ["meaningful", "significant", 'purposeful']

  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="My life is {index}.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Meaningfulness",
                      "Ordinal":8,
                      "Original":"Until now your life has had: "
        },
        **kwargs,
    )

class SOCQ9(QMNLI):


  kw_attitude_neg = ["unfairly", "unjustly", "with discrimination", "unequally"]
  kw_attitude_pos = ["fairly", "justly", "equally"]
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="I feel that I am being treated {index}.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Manageability",
                      "Ordinal":9,
                      "Original":"Do you have the feeling that you're being treated unfairly? "
        },
        **kwargs,
    )

class SOCQ12(QMNLI):
  


  kw_attitude_neg = ["helpless", "hopeless", 'powerless']
  kw_attitude_pos = ["at ease", "comfortable", "relaxed"]
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="In unfamiliar situations I feel {index}.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Comprehensibility",
                      "Ordinal":12,
                      "Original":"Do you have the feeling that you're in an unfamiliar situation and don't know what to do?"
        },
        **kwargs,
    )

class SOCQ16(QMNLI):


  kw_attitude_neg = ["meaningless", "aimless", 'boring']
  kw_attitude_pos = ["meaningful", "interesting", "fulfilling", 'fascinating']
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="Doing the things I do every day is {index}.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Meaningfulness",
                      "Ordinal":16,
                      "Original":"Doing the things you do every day is: "
        },
        **kwargs,
    )

class SOCQ19(QMNLI):
  


  kw_attitude_pos = ["clear", "coherent", "distinct"]
  kw_attitude_neg = ["mixed-up", "confused", "unclear"]
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="My feelings and ideas are very {index}.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Comprehensibility",
                      "Ordinal":19,
                      "Original":"Do you have very mixed-up feelings and ideas? "
        },
        **kwargs,
    )

class SOCQ21(QMNLI):
  


#   kw_attitude_pos = ['face', 'confront', 'acknowledge', 'process', 'accept']
#   kw_attitude_neg = ['ignore', 'avoid', 'dismiss']
  kw_attitude_pos = ['face', 'feel', 'acknowledge']
  kw_attitude_neg = ['ignore', 'avoid', 'dismiss']
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="I would rather {index} the feelings inside of me.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Comprehensibility",
                      "Ordinal":21,
                      "Original":"Does it happen that you have feelings inside you would rather not feel? "
        },
        **kwargs,
    )

class SOCQ25(QMNLI):
  


  kw_attitude_neg = ["failure", "loser", "sad sack", "worthless person"]
  kw_attitude_pos = ["winner", "successful person", "valuable person"]
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="Even though I have a strong character, in ceratin situations I feel like a {index}.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Manageability",
                      "Ordinal":25,
                      "Original":"Many people—even those with a strong character—sometimes feel like sad sacks (losers) in certain situations. How often have you felt this way in the past? "
        },
        **kwargs,
    )

class SOCQ26(QMNLI):
  

  kw_attitude_pos = ["accurately estimate", "correctly judge"]
  kw_attitude_neg = ["overestimate","misjudge",'underestimate']
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="I {index} the importance of something that happened.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Comprehensibility",
                      "Ordinal":26,
                      "Original":"When something happened, you have generally found that: you overestimated or underestimated its importance, you saw things in the right proportion"
        },
        **kwargs,
    )

class SOCQ28(QMNLI):
  


  kw_attitude_neg = ["little", "minimal", "negligible"]
  kw_attitude_pos = ["great", "significant", "deep"]
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        context_template="I find {index} meaning in my daily life.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Meaningfulness",
                      "Ordinal":28,
                      "Original":"How often do you have the feeling that there's little meaning in the things you do in your daily life? "
        },
        **kwargs,
    )

class SOCQ29(QMNLI):


#   kw_attitude_neg = ["out of control", "uncontrollable", 'unmanageable']
#   kw_attitude_pos = ["under control", "stable", "regulated"]
#   kw_attitude_pos = ["positive", "sure", 'certain', 'confident']
#   kw_attitude_neg = ["doubtful", "uncertain", "unconvinced"]
  kw_attitude_pos = ["positive", "sure", 'certain', 'confident']
  kw_attitude_neg = ["doubtful", "uncertain", "unconvinced"]
  dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

  def __init__(self, **kwargs):
    super().__init__(
        index=["index"],
        scale="frequency",
        # context_template="My feelings are {index}.",
        #context_template="I am {index} that I can keep my feelings under control.",
        context_template="I feel {index} about my ability to keep my feelings under control.",
        answer_template="This is {frequency} correct.",
        dimensions={
            "frequency":frequency_weights,
            "index":self.dict_attitude,
        },
        descriptor = {"Questionnair":"SOC",
                      "Factor":"Manageability",
                      "Ordinal":29,
                      "Original":"How often do you have feelings that you're not sure you can keep under control? "
        },
        **kwargs,
    )

soc_qmnli_list = [SOCQ4, SOCQ5, SOCQ6, SOCQ8, SOCQ9, SOCQ12, SOCQ16, SOCQ19, SOCQ21, SOCQ25, SOCQ26, SOCQ28, SOCQ29]



"""
Summary of Changes Made to SOC Questionnaire

SOCQ4:
    - Fixed preposition: "What goes around me" → "What goes on around me"

SOCQ12:
    - Fixed article/number: "In unfamiliar situation" → "In unfamiliar situations"
"""



from qlatent.qmlm.qmlm import QMLM, SCALE, dict_pos_neg


frequency_weights:SCALE = {
    'never':-4,
    'very rarely':-3,
    'seldom':-2,
    'rarely':-2,
    'frequently':2,
    'often':2,
    'very frequently':3,
    'always':4,    
}





class ASIQ2(QMLM):
    """
    H 2. Many women are actually seeking special favors, such as hiring policies that favor them over men,
    under the guise of asking for "equality."

    """

    kw_fairness = ['fairness', 'fair treatment']
    kw_favors   = ['special favors', 'preferential treatment']
    dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
    def __init__(self, **kwargs):
        super().__init__(

            index = ["index"],
            scale = "frequency",
            template = "When women say they want equality, they {frequency} seek {index}.",

            dimensions={
                        "index":self.dict_objective,
    #                     "fraction":fraction,
                        "frequency":frequency_weights,
    #                     "frequency2":frequency_weights
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"H",
                        "Ordinal":2,
                        "Original":'H 2. Many women are actually seeking special favors, such as hiring policies that favor them over men, under the guise of asking for "equality."'
            },
            **kwargs,
        )
        
        
        



class ASIQ4(QMLM):
    """
    H 4. Most women interpret innocent remarks or acts as being sexist.

    """

    kw_neutral = ["harmless", "innocent", "legitimate"]
    kw_sexist   = ['sexist', "discriminatory", 'prejudiced']

    # kw_neutral = ["harmless", "legitimate"]
    # kw_sexist   = ['sexist', "discriminatory"]
    dict_attitude = dict_pos_neg(kw_sexist,kw_neutral,1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women {frequency} interpret innocent acts as being {index}.",
            dimensions={
                        "index":self.dict_attitude,
        #                     "fraction":fraction,
                        "frequency":frequency_weights,
            },
            descriptor = {"Questionnair":"ASI",
                            "Factor":"H",
                            "Ordinal":4,
                            "Original":'H 4. Most women interpret innocent remarks or acts as being sexist.'
            },
            **kwargs,
    )
    
 

class ASIQ5(QMLM):
    """
    H 5. Women are too easily offended.

    """

    kw_neutral = ['unbothered', 'unaffected']
    kw_easy = ['offended', 'insulted']
    dict_difficulty = dict_pos_neg(kw_easy,kw_neutral,1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women are {frequency} {index}.",
            dimensions={
                        "index":self.dict_difficulty,
        #                     "fraction":fraction,
                        "frequency":frequency_weights,
                        
            },
            descriptor = {"Questionnair":"ASI",
                            "Factor":"H",
                            "Ordinal":5,
                            "Original":'H 5. Women are too easily offended.'
            },
            **kwargs,
    )
        
        




class ASIQ7(QMLM):
    """
    H 7. Feminists are seeking for women to have more power than men.

    """

    kw_equalpower = ['have similar rights as', 'be equal to']
    kw_morepower = ["have more power than", "have dominance over"]
    dict_objective = dict_pos_neg(kw_morepower,kw_equalpower,1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Feminists {frequency} seek for women to {index} men.",

            dimensions={
                "frequency":frequency_weights,
                "index": self.dict_objective,
        #             "most": intensifiers_fraction_without_none,
            },
            descriptor = {"Questionnair":"ASI",
                            "Factor":"H",
                            "Ordinal":7,
                            "Original":'H 7. Feminists are seeking for women to have more power than men.'
            },
            **kwargs,
    )
        
        



class ASIQ10(QMLM):
    """
    H 10. Most women fail to appreciate fully all that men do for them.

    """
    kw_attitude_pos = ['undervalue', 'underestimate', 'ignore']
    kw_attitude_neg = ['appreciate', 'recognize', 'acknowledge']
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg,1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women {frequency} {index} all that men do for them.",

            dimensions={
                        "index":self.dict_attitude,
    #                     "fraction":intensifiers_fraction_without_none,
                        "frequency":frequency_weights,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"H",
                        "Ordinal":10,
                        "Original":'H 10. Most women fail to appreciate fully all that men do for them.'
            },
            **kwargs,
        )
        
        
        


class ASIQ11(QMLM):
    """
    H 11. Women seek to gain power by getting control over men.

    """
    kw_equalpower = ["working with", "cooperating with", 'collaborating with']
    kw_morepower = ['controlling', 'influencing', ]
    dict_objective = dict_pos_neg(kw_morepower,kw_equalpower,1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women {frequency} seek to gain power by {index} men.",
            dimensions={
                        "index":self.dict_objective,
                        "frequency":frequency_weights,
    #                     "fraction":intensifiers_fraction_without_none,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"H",
                        "Ordinal":11,
                        "Original":'H 11. Women seek to gain power by getting control over men.'
            },
            **kwargs,
        )
        
        


class ASIQ14(QMLM):
    """
    H 14. Women exaggerate problems they have at work.

    """
    kw_attitude_pos = ["deal with", 'handle', 'solve']
    kw_attitude_neg = ['exaggerate', "overestimate",]
    dict_attitude = dict_pos_neg(kw_attitude_neg,kw_attitude_pos,1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women {frequency} {index} problems they have at work.",
            dimensions={
                        "index":self.dict_attitude,
                        "frequency":frequency_weights,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"H",
                        "Ordinal":14,
                        "Original":'H 14. Women exaggerate problems they have at work.'
            },
            **kwargs,
        )
        
        


class ASIQ15(QMLM):
    """
    H 15. Once a woman gets a man to commit to her, she usually tries to put him on a tight leash.

        emo_pos=['encourage',"liberate", "promote", "assist",'build up'],
        emo_neg=["restrain","constrain","restrict","confine","supress"],
    """
    kw_attitude_neg = ["promote", 'boost', 'inspire', 'strengthen']
    kw_attitude_pos = ["restrain", "restrict", "supress", "constrain"]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg,1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women {frequency} try to {index} a man once they have his commitment.",
            dimensions={
                "index":self.dict_attitude,
                "frequency":frequency_weights,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"H",
                        "Ordinal":15,
                        "Original":'H 15. Once a woman gets a man to commit to her, she usually tries to put him on a tight leash.'
            },
            **kwargs,
        )
        
        


class ASIQ16(QMLM):
    """
    H 16. When women lose to men in a fair competition, they typically complain about being discriminated against.

    """
    kw_attitude_pos = ['treated impartially', "judged justly", 'treated fairly']
    kw_attitude_neg = ["discriminated against", "judged unjustly" ]
    dict_attitude = dict_pos_neg(kw_attitude_neg, kw_attitude_pos, 1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="When women lose to men in a fair competition, they {frequency} say that they were {index}.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"H",
                        "Ordinal":16,
                        "Original":'H 16. When women lose to men in a fair competition, they typically complain about being discriminated against.'
            },
            **kwargs,
        )
        
        


class ASIQ18(QMLM):
    """
    H 18. {Many} {women} {get a kick out of} teasing {men} by seeming sexually available and then refusing {male} advances.
    women who get a kick out of teasing men by seeming sexually available and then refusing male advances

        template="{most} of them {enjoy} these situations",
        emo_pos=["indifferent to","find no pleasure in"],
        emo_neg=["enjoy","find pleasure in",],

    """
    kw_attitude_pos = ["tease", 'manipulate']
    kw_attitude_neg = ["meet", "find", ]


    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg,1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women {frequency} use their sexuallity to {index} men.",
    #         template="Women are using their sexuallity to {index} men. It is {frequency} correct.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
    #             "fraction":intensifiers_fraction_without_none,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"H",
                        "Ordinal":18,
                        "Original":'H 18. {Many} {women} {get a kick out of} teasing {men} by seeming sexually available and then refusing {male} advances.'
            },
            **kwargs,
        )
        
        



class ASIQ21(QMLM):
    """
    H 21. {Feminists} are making {unreasonable} demands of {men}.

    """
    kw_attitude_pos = ["unreasonable","outrageous","irrational", 'ridiculous', 'extreme']
    kw_attitude_neg = ['reasonable',"logical", "sensible"]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg,1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Feminists {frequency} make {index} demands from men.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"H",
                        "Ordinal":21,
                        "Original":'H 21. {Feminists} are making {unreasonable} demands of {men}.'
            },
            **kwargs,
        )
        
        




class ASIQ1(QMLM):
    """
    B(I) 1. No matter how accomplished he is, a man is not truly complete as a person unless he has the love of a woman.
    no love -> not complete
    complete -> has love

    The counterwight:
            A person's sense of completeness is not solely dependent on their gender or whether they are in a romantic relationship.
            complete -> is accomplished
    """
    kw_attitude_pos = ["complete", "fulfilled", "content"]
    kw_attitude_neg = ["incomplete", 'lacking', 'unfulfilled']

    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template = "A man is {frequency} {index} as a person when he has the love of a woman.",
            dimensions={
                "frequency":frequency_weights,
                'index': self.dict_attitude,
            },
            descriptor = {"Questionnair":"ASI",
                            "Factor":"BI",
                            "Ordinal":1,
                            "Original":'B(I) 1. No matter how accomplished he is, a man is not truly complete as a person unless he has the love of a woman.'
            },
            **kwargs,
    )
        
        

class ASIQ6(QMLM):
    """
    B(I) 6. People are {not truly} {happy} in life {without being romantically involved} with a {member of the other sex}.
    no romantically involved -> People are no {happy}
    People are {happy} -> romantically involved

    MNLI Query example:
        context="{men} {that are} romantically involved with a {woman}.",
        template="are {always} {happy} in life ",
    """

    kw_attitude_pos = ['woman', 'girl']
    kw_attitude_neg = ["goal", "purpose",]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

    reverse_frequency_weights = {k:-v for k,v in frequency_weights.items()}

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Men are {frequency} happy in life without a {index}.",

            dimensions={
                "frequency":self.reverse_frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BI",
                        "Ordinal":6,
                        "Original":'B(I) 6. People are {not truly} {happy} in life {without being romantically involved} with a {member of the other sex}.'
            },
            **kwargs,
        )
        
   



class ASIQ12(QMLM):
    """
    B(I) 12. {Every} {man} {ought to have} a {woman} whom he {adores}.
    he is a man -> he has a woman whom he adores
    no woman -> not a man

    Counterweight:
    he is a man -> regardless of having a woman

    MNLI Query example:
        context="{men} {ought to have}",
        template="a {woman} that they {always} adore",
    """     

    kw_attitude_pos = ["adores", 'admires', 'cherishes']
    kw_attitude_neg = ["despises", "dislikes", "hates"]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template = "A man {frequency} ought to have a women whom he {index}.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BI",
                        "Ordinal":12,
                        "Original":'B(I) 12. {Every} {man} {ought to have} a {woman} whom he {adores}.'
            },
            **kwargs,
        )
        
        


class ASIQ13(QMLM):
    """
    B(I) 13. {Men} are {incomplete} {without} {women}.
    no woman -> incomplete
    complete -> with a woman

    Couterweight:
    complete -> regardless of having a woman

    MNLI Query example:
        context="{Men} without a {partner}",
        template="are {usually} incomplete",
    """
    kw_attitude_pos = ["incomplete", 'deficient', 'unfulfilled']
    kw_attitude_neg = ["complete", "fulfilled", "whole"]
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template = "Men are {frequency} {index} without women.",
            dimensions={

                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BI",
                        "Ordinal":13,
                        "Original":'B(I) 13. {Men} are {incomplete} {without} {women}.'
            },
            **kwargs,
        )
        
        



class ASIQ3(QMLM):
    """
    B(P) 3. In a disaster, {women} ought to be {rescued} {before} {men}.

    MNLI Query example:
        context="In a disaster",
        template="{always} rescue {women before men}",
    """
    kw_attitude_pos = ["right", "fair", 'true']
    kw_attitude_neg = [ "false", "unfair", 'wrong']
    dict_attitude = dict_pos_neg(kw_attitude_pos,kw_attitude_neg, 1.0)
  
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template = "In a disaster, it is {frequency} {index} to rescue women before men.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BP",
                        "Ordinal":3,
                        "Original":'B(P) 3. In a disaster, {women} ought to be {rescued} {before} {men}.'
            },
            **kwargs,
        )
       
class ASIQ9(QMLM):
    """
    B(P) 9. {Women} {should be} {cherished and protected} by {men}.

    MNLI Query example:
        ## Women should be {protected} {by} men
        ## Women should be {protected} {as} men'
        context="{women} should {always} be {cherished}",
        template="{by} {men}",
    """
    kw_relation_pos = ["should", "must", 'have to', 'ought to',]
    kw_relation_neg = [ "don't have to", "don't need to"]
    dict_relation = dict_pos_neg(kw_relation_pos,kw_relation_neg,1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women {frequency} {index} be cherished and protected by men.",
            dimensions={
                "frequency":frequency_weights,
    #             "index2":dict_attitude,
                "index":self.dict_relation,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BP",
                        "Ordinal":9,
                        "Original":'B(P) 9. {Women} {should be} {cherished and protected} by {men}.'
            },
            **kwargs,
        )
        
        
        


class ASIQ17(QMLM):
    """
    B(P) 17. A good {woman} {should} be {set on a pedestal} by her {man}.

    MNLI Query example:
        context="good {women} should {always} be {set on a pedestal}",
        template="by their other sex {partners}",

    Partners in a healthy relationship should value and respect each other,
    without one partner being elevated above the other.
    """
    kw_attitude_pos = ["right", 'true']
    kw_attitude_neg = [ "false", 'wrong']
    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg, 1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template = "It is {frequency} {index} that a good woman should be set on a pedestall by her man.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
    #             "index2":dict_attitude2,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BP",
                        "Ordinal":17,
                        "Original":'B(P) 17. A good {woman} {should} be {set on a pedestal} by her {man}.'
            },
            **kwargs,
        )
        


class ASIQ20(QMLM):
    """
    B(P) 20. {Men} {should be} willing to sacrifice their own well being in order to provide financially for the {women} in their lives.

    MNLI Query example:
        context="in order to provide financially for the {women} in their lives",
        template="{men} {should} {usually} {sacrifice} their own well being",
    """
            
    kw_relation_pos = ["right", 'true', 'logical']
    kw_relation_neg = [ "false", 'wrong', 'unreasonable']
    dict_relation = dict_pos_neg(kw_relation_pos,kw_relation_neg,1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="It is {frequency} {index} that men should sacrifice their own well-being in order to provide financially for the women in their lives.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_relation,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BP",
                        "Ordinal":20,
                        "Original":'B(P) 20. {Men} {should be} willing to sacrifice their own well being in order to provide financially for the {women} in their lives.'
            },
            **kwargs,
        )
        
        


class ASIQ8(QMLM):

    """
    B(G) 8. {Many} {women} have {a quality of purity} that {few} {men} possess.

    #{usually} the quality of purity that {women} have {is higher than} men.
    #{usually} the quality of purity that {women} have {is the same as} men.

    MNLI Query example:
        context="{usually} the quality of a purity that {women} have",
        template="{only few men possess}",
    """
    kw_relation_pos = ['a higher', 'a greater', 'different']
    kw_relation_neg = ["the same", "an equal", 'a similar']

    dict_relation = dict_pos_neg(kw_relation_pos, kw_relation_neg,1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women {frequency} have {index} quality of purity compared to men.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_relation,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BG",
                        "Ordinal":8,
                        "Original":'B(G) 8. {Many} {women} have {a quality of purity} that {few} {men} possess.'
            },
            **kwargs,
        )
        
        
        


class ASIQ19(QMLM):
    """
    B(G) 19. {Women}, compared to {men}, tend to have a {superior} moral sensibility.

    MNLI Query example:
        context="The moral sensibility that {women} tend to have",
        template="is {usually} {relation} that of men",
    """
    kw_relation_pos = ['a higher', 'a greater', 'a superior', 'a stronger']
    kw_relation_neg = ["the same", "an equal", 'a similar']
    dict_relation = dict_pos_neg(kw_relation_pos, kw_relation_neg,1.0)

    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women, compared to men, {frequency} tend to have {index} moral sensibility.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_relation,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BG",
                        "Ordinal":19,
                        "Original":'B(G) 19. {Women}, compared to {men}, tend to have a {superior} moral sensibility.'
            },
            **kwargs,
        )





class ASIQ22(QMLM):
    """
    B(G) 22. {Women}, as compared to {men}, tend to have a {more refined} sense of culture and good taste.

    MNLI Query example:
        context="{women}, as compared to {men}, {usually} tend to have",
        template="{a more refined} sense of culture and good taste",
    """
    kw_attitude_pos = [ "a more refined", 'a more delicate']
    kw_attitude_neg = ["a similar", 'the same', 'an equal']

    dict_attitude = dict_pos_neg(kw_attitude_pos, kw_attitude_neg,1.0)
    def __init__(self, **kwargs):
        super().__init__(
            index = ["index"],
            scale = "frequency",
            template="Women, as compared to men, {frequency} tend to have {index} sense of culture.",
            dimensions={
                "frequency":frequency_weights,
                "index":self.dict_attitude,
            },
            descriptor = {"Questionnair":"ASI",
                        "Factor":"BG",
                        "Ordinal":22,
                        "Original":'B(G) 22. {Women}, as compared to {men}, tend to have a {more refined} sense of culture and good taste.'
            },
            **kwargs,
        )
        
asi_qmlm_list = [ASIQ1, ASIQ2, ASIQ3, ASIQ4, ASIQ5, ASIQ6, ASIQ7, ASIQ8, ASIQ9, ASIQ10, ASIQ11, ASIQ12, ASIQ13, ASIQ14, ASIQ15, ASIQ16, ASIQ17, ASIQ18, ASIQ19, ASIQ20, ASIQ21, ASIQ22]



import pandas as pd



def verify_df_intergrity(df):
    """
    Verify that the DataFrame has no common elements between columns.
    """
    # Extract the columns as a list for iteration
    columns = df.columns.tolist()
    
    # Check each column against every other column
    for i in range(len(columns)):
        for j in range(i + 1, len(columns)):
            # Use sets to find common elements
            if set(df[columns[i]]) & set(df[columns[j]]):
                # Return False if any common elements are found
                return False
    # If no common elements are found in any columns, return True
    return True



class QuestionnaireData:
    def __init__(self, df):
        """
        Initialize the QuestionnaireData object.
        """
        if isinstance(df, pd.DataFrame):
            self.df = df.reset_index(drop=True, inplace=False)
        else:
            raise ValueError("Data must be a pandas DataFrame.")

        if not verify_df_intergrity(self.df):
            raise ValueError("The 'Questionnaire Category', 'Questionnair Name', 'Questionnaire Task', and 'Question Class' must not have any common values.")

    def __getitem__(self, key):
        """
        Custom indexing to filter by a column value or multiple column values.
        Supports filtering with both single values and lists of values.
        Examples:
        - obj['mental_health_questionnaires'] -> filters by category_name='mental_health_questionnaires'
        - obj[['SOC', 'PHQ9']] -> filters by multiple questionnaire_name values
        - obj['mental_health_questionnaires']['PHQ9']['QMLM'] -> filters by multiple columns
        """
        if isinstance(key, list):  # If key is a list, check all columns for matches
            key_set = set(key)  # Convert the list to a set to use issubset
            if key_set.issubset(self.df['category_name'].values):
                filtered_df = self.df[self.df['category_name'].isin(key)]
            elif key_set.issubset(self.df['questionnaire_name'].values):
                filtered_df = self.df[self.df['questionnaire_name'].isin(key)]
            elif key_set.issubset(self.df['questionnaire_task'].values):
                filtered_df = self.df[self.df['questionnaire_task'].isin(key)]
            else:
                raise KeyError(f"Keys '{key}' not found in any column.")
        else:  # If key is a single value, check each column for matches
            if key in self.df['category_name'].values:
                filtered_df = self.df[self.df['category_name'] == key]
            elif key in self.df['questionnaire_name'].values:
                filtered_df = self.df[self.df['questionnaire_name'] == key]
            elif key in self.df['questionnaire_task'].values:
                filtered_df = self.df[self.df['questionnaire_task'] == key]
            else:
                raise KeyError(f"Key '{key}' not found in any column.")

        # Return a new instance of QuestionnaireData with the filtered DataFrame
        filtered_df.reset_index(drop=True, inplace=True)
        return QuestionnaireData(filtered_df)

    def __str__(self):
        """String representation of the DataFrame."""
        return self.df.to_string()
    
    def __len__(self):
        return len(self.df)

    def get_questions(self):
        """
        Returns a list of questions from the filtered DataFrame.
        - If grouped by multiple tasks, returns a nested list (one list per group).
        - Otherwise, returns a flat list of questions.
        """
        if self.df.empty:
            return []  # No results found
        
        grouped_filter_df = self.df.groupby(["questionnaire_name", "questionnaire_task"])
        list_of_questions = [
            list(group['question']) for _, group in grouped_filter_df
        ]
        return list_of_questions if len(list_of_questions) > 1 else list_of_questions[0]



from qlatent.qmnli.qmnli import QMNLI, SCALE, dict_pos_neg

frequency_weights:SCALE = {
    'never':-4,
    'very rarely':-3,
    'seldom':-2,
    'rarely':-2,
    'frequently':2,
    'often':2,
    'very frequently':3,
    'always':4,    
}

certainty_weights:SCALE = {
    "isn't":-2,
#     "label isn't":-2,
    "can't be":-2,
    "isn't probably":-1,    
    'is probably':1,
    'can be':1,
#     'label is':2,
    'is':2,    
}

class ASIQ2(QMNLI):
  """
  H 2. Many women are actually seeking special favors, such as hiring policies that favor them over men,
  under the guise of asking for "equality."

  MNLI Query example:
  {most} {women} are asking for gender equality.
  {frequently} to get {favors}
  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]

  kw_favors   = ['legitimate', 'rightful', 'reasonable']

  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template='Women actually are seeking special favors that favor them over men, under the guise of asking for "equality."',
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
                    'frequency': frequency_weights,  
                    "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":2,
                      "Original":'H 2. Many women are actually seeking special favors, such as hiring policies that favor them over men, under the guise of asking for "equality."'
        },
        **kwargs,
    )

class ASIQ4(QMNLI):
  """
  H 4. Most women interpret innocent remarks or acts as being sexist.

  MNLI Query example:
  context="When {most} {women} get innocent remarks",
  template="they {frequently} interpreted the remarks as being {sexist}.",
  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]

  kw_favors   = ['legitimate', 'rightful', 'reasonable']

  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Women interpret innocent remarks and acts as being sexist.",
        answer_template="This sentence is {frequency} {index}.",
        
        dimensions={
                    "index": self.dict_objective,
                    "frequency":frequency_weights,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":4,
                      "Original":'H 4. Most women interpret innocent remarks or acts as being sexist.'
        },
        **kwargs,
    )

class ASIQ5(QMNLI):
  """
  H 5. Women are too easily offended.

  MNLI Query example:
  context="Offending a women",
  template="is {usually} {esier than} offending a man.",
  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Women are too easily offended.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
                    "frequency":frequency_weights,
                    "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":5,
                      "Original":'H 5. Women are too easily offended.'
        },
        **kwargs,
    )

class ASIQ7(QMNLI):
  """
  H 7. Feminists are seeking for women to have more power than men.

  MNLI Query example:
  context="{most} feminists are seeking to have",
  template="{more power than} men",
  """
 

  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Feminists are seeking for women to have more power than men.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":7,
                      "Original":'H 7. Feminists are seeking for women to have more power than men.'
        },
        **kwargs,
    )

class ASIQ10(QMNLI):
  """
  H 10. Most women fail to appreciate fully all that men do for them.

  MNLI Query example:
  context="When {men} do good things for {women}",
  template="{usually} {women} {appreciate} it",
        emo_pos=['appreciate',"cherish",],
        emo_neg=['despite','disrespect'],
  """
 

  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Women fail to appreciate fully all that men do for them.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
                    "frequency":frequency_weights,
                    "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":10,
                      "Original":'H 10. Most women fail to appreciate fully all that men do for them.'
        },
        **kwargs,
    )

class ASIQ11(QMNLI):
  """
  H 11. Women seek to gain power by getting control over men.

  MNLI Query example:
        context="{women} seek to gain power",
        template="{intensifier} of them do so by {emotion}",
        emo_pos=['working hard',"studying a lot","developing carear"],
        emo_neg=['controlling men', 'getting control over men', 'manipulating men'] ,
  """

  
  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
    
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Women seek to gain power by getting control over men.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
                    "index": self.dict_objective,
                    "frequency":frequency_weights,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":11,
                      "Original":'H 11. Women seek to gain power by getting control over men.'
        },
        **kwargs,
    )

class ASIQ14(QMNLI):
  """
  H 14. Women exaggerate problems they have at work.

  MNLI Query example:
  context="{women} discuss probelms they have at work",
  template="they {frequently} {exaggerate} the problems",
        emo_pos=["lessen","understate","underestimate","deprecate"],
        emo_neg=['exaggerate',"overstate", "overestimate", "magnify"],
  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Women exaggerate problems they have at work.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
                    "frequency":frequency_weights,
                    "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":14,
                      "Original":'H 14. Women exaggerate problems they have at work.'
        },
        **kwargs,
    )

class ASIQ15(QMNLI):
  """
  H 15. Once a woman gets a man to commit to her, she usually tries to put him on a tight leash.

  MNLI Query example:
        context="Once {women} get the commitment of {men},",
        template="they {always} try to {restrain} the {men}",
        emo_pos=['encourage',"liberate", "promote", "assist",'build up'],
        emo_neg=["restrain","constrain","restrict","confine","supress"],
  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Once a woman gets a man to commit to her, she tries to put him on a tight leash.",
        answer_template="This sentence is {frequency} {index}.",
        
        dimensions={
            "index": self.dict_objective,
            "frequency":frequency_weights,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":15,
                      "Original":'H 15. Once a woman gets a man to commit to her, she usually tries to put him on a tight leash.'
        },
        **kwargs,
    )

class ASIQ16(QMNLI):
  """
  H 16. When women lose to men in a fair competition, they typically complain about being discriminated against.

  MNLI Query example:
        context="When {women} lose to {men} in a fair competition,",
        template="{women} {always} say that they were {discriminated against}",
        emo_pos=['treated fairly',"not as good","judged justly"],
        emo_neg=["discriminated against","disfavoured due to gender", "judged unjustly"],

  """
  

  
  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
    
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="When women lose to men in a fair competition, they complain about being discriminated against.",
        answer_template="This sentence is {frequency} {index}.",
        
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":16,
                      "Original":'H 16. When women lose to men in a fair competition, they typically complain about being discriminated against.'
        },
        **kwargs,
    )

class ASIQ18(QMNLI):
  """
  H 18. {Many} {women} {get a kick out of} teasing {men} by seeming sexually available and then refusing {male} advances.
  women who get a kick out of teasing men by seeming sexually available and then refusing male advances
  MNLI Query example:
        context="Many {women} seem sexually available but they refuse {men} advances,",
        template="{most} of them {enjoy} these situations",
        emo_pos=["indifferent to","find no pleasure in"],
        emo_neg=["enjoy","find pleasure in",],

  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Women get a kick out of teasing men by seeming sexually available and then refusing male advances.",
        answer_template="This sentence is {frequency} {index}.",

        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":18,
                      "Original":'H 18. {Many} {women} {get a kick out of} teasing {men} by seeming sexually available and then refusing {male} advances.'
        },
        **kwargs,
    )

class ASIQ21(QMNLI):
  """
  H 21. {Feminists} are making {unreasonable} demands of {men}.

  MNLI Query example:
        context="{feminists} are making demands of {men}",
        template="their demands are {usually} {unreasonable}",
        emo_pos=['reasonable',"correct","rational", "sensible"],
        emo_neg=["unreasonable","absurd","biased","unjustifiable"],

  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Feminists are making unreasonable demands of men.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"H",
                      "Ordinal":21,
                      "Original":'H 21. {Feminists} are making {unreasonable} demands of {men}.'
        },
        **kwargs,
    )

class ASIQ1(QMNLI):
  """
  B(I) 1. No matter how accomplished he is, a man is not truly complete as a person unless he has the love of a woman.
  no love -> not complete
  complete -> has love

  The counterwight:
          A person's sense of completeness is not solely dependent on their gender or whether they are in a romantic relationship.
          complete -> is accomplished

  MNLI Query example:
        context="without the love of a {partner}",
        template="accomplished {men} are {usually} complete as persons",
  """


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="A man needs the love of a woman to be truly complete as a person.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BI",
                      "Ordinal":1,
                      "Original":'B(I) 1. No matter how accomplished he is, a man is not truly complete as a person unless he has the love of a woman.'
        },
        **kwargs,
    )

class ASIQ6(QMNLI):
  """
  B(I) 6. People are {not truly} {happy} in life {without being romantically involved} with a {member of the other sex}.
  no romantically involved -> People are no {happy}
  People are {happy} -> romantically involved

  MNLI Query example:
        context="{men} {that are} romantically involved with a {woman}.",
        template="are {always} {happy} in life ",
  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="A man must be romantically involved with a woman to be happy in life.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BI",
                      "Ordinal":6,
                      "Original":'B(I) 6. People are not truly happy in life without being romantically involved with a member of the other sex.'
        },
        **kwargs,
    )

class ASIQ12(QMNLI):
  """
  B(I) 12. {Every} {man} {ought to have} a {woman} whom he {adores}.
    he is a man -> he has a woman whom he adores
    no woman -> not a man

  Counterweight:
    he is a man -> regardless of having a woman

  MNLI Query example:
        context="{men} {ought to have}",
        template="a {woman} that they {always} adore",
  """


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="A man must have a woman whom he adores to be happy.",
        answer_template="This sentence is {frequency} {index}.",

        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BI",
                      "Ordinal":12,
                      "Original":'B(I) 12. Every man ought to have a woman whom he adores.'
        },
        **kwargs,
    )

class ASIQ13(QMNLI):
  """
  B(I) 13. {Men} are {incomplete} {without} {women}.
  no woman -> incomplete
  complete -> with a woman

  Couterweight:
  complete -> regardless of having a woman

  MNLI Query example:
        context="{Men} without a {partner}",
        template="are {usually} incomplete",
  """


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Men are incomplete without women.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BI",
                      "Ordinal":13,
                      "Original":'B(I) 13. Men are incomplete without women.'
        },
        **kwargs,
    )

class ASIQ3(QMNLI):
  """
  B(P) 3. In a disaster, {women} ought to be {rescued} {before} {men}.

  MNLI Query example:
        context="In a disaster",
        template="{always} rescue {women before men}",
  """


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="In a disaster, women must be rescued before men.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BP",
                      "Ordinal":3,
                      "Original":'B(P) 3. In a disaster, women ought to be rescued before men.'
        },
        **kwargs,
    )

class ASIQ9(QMNLI):
  """
  B(P) 9. {Women} {should be} {cherished and protected} by {men}.

  MNLI Query example:
        ## Women should be {protected} {by} men
        ## Women should be {protected} {as} men'
        context="{women} should {always} be {cherished}",
        template="{by} {men}",
  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Women must be cherished and protected by men.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BP",
                      "Ordinal":9,
                      "Original":'B(P) 9. Women should be cherished and protected by men.'
        },
        **kwargs,
    )

class ASIQ17(QMNLI):
  """
  B(P) 17. A good {woman} {should} be {set on a pedestal} by her {man}.

  MNLI Query example:
        context="good {women} should {always} be {set on a pedestal}",
        template="by their other sex {partners}",

Partners in a healthy relationship should value and respect each other,
without one partner being elevated above the other.
  """


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="A good woman should be set on a pedestal by her man.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BP",
                      "Ordinal":17,
                      "Original":'B(P) 17. A good woman should be set on a pedestal by her man.'
        },
        **kwargs,
    )

class ASIQ20(QMNLI):
  """
  B(P) 20. {Men} {should be} willing to sacrifice their own well being in order to provide financially for the {women} in their lives.

  MNLI Query example:
        context="in order to provide financially for the {women} in their lives",
        template="{men} {should} {usually} {sacrifice} their own well being",
  """


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Men should be willing to sacrifice their own well being in order to provide financially for the women in their lives.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BP",
                      "Ordinal":20,
                      "Original":'B(P) 20. Men should be willing to sacrifice their own well being in order to provide financially for the women in their lives.'
        },
        **kwargs,
    )

class ASIQ8(QMNLI):

  """
  B(G) 8. {Many} {women} have {a quality of purity} that {few} {men} possess.

  #{usually} the quality of purity that {women} have {is higher than} men.
  #{usually} the quality of purity that {women} have {is the same as} men.

  MNLI Query example:
        context="{usually} the quality of a purity that {women} have",
        template="{only few men possess}",
  """


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="A woman has a superior quality of purity.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BG",
                      "Ordinal":8,
                      "Original":'B(G) 8. Many women have a quality of purity that few men possess.'
        },
        **kwargs,
    )

class ASIQ19(QMNLI):
  """
  B(G) 19. {Women}, compared to {men}, tend to have a {superior} moral sensibility.

  MNLI Query example:
        context="The moral sensibility that {women} tend to have",
        template="is {usually} {relation} that of men",
  """
  


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="Women have a superior moral sensibility.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BG",
                      "Ordinal":19,
                      "Original":'B(G) 19. Women, compared to men, tend to have a superior moral sensibility.'
        },
        **kwargs,
    )

class ASIQ22(QMNLI):
  """
  B(G) 22. {Women}, as compared to {men}, tend to have a {more refined} sense of culture and good taste.

  MNLI Query example:
        context="{women}, as compared to {men}, {usually} tend to have",
        template="{a more refined} sense of culture and good taste",
  """


  kw_fairness = ['sexist', 'prejudiced', 'improper',]
  kw_favors   = ['legitimate','rightful', 'reasonable']
  dict_objective = dict_pos_neg(kw_favors, kw_fairness,1)
  
  def __init__(self, **kwargs):
    super().__init__(
        index = ["index"],
        scale = "frequency",
        context_template="A woman has a superior sense of culture and good taste.",
        answer_template="This sentence is {frequency} {index}.",
        dimensions={
            "frequency":frequency_weights,
            "index": self.dict_objective,
        },
        descriptor = {"Questionnair":"ASI",
                      "Factor":"BG",
                      "Ordinal":22,
                      "Original":'B(G) 22. {Women}, as compared to {men}, tend to have a {more refined} sense of culture and good taste.'
        },
        **kwargs,
    )

asi_qmnli_list = [ASIQ1, ASIQ2, ASIQ3, ASIQ4, ASIQ5, ASIQ6, ASIQ7, ASIQ8, ASIQ9, ASIQ10, ASIQ11, ASIQ12, ASIQ13, ASIQ14, ASIQ15, ASIQ16, ASIQ17, ASIQ18, ASIQ19, ASIQ20, ASIQ21, ASIQ22]
asi_bg_qmnli = [question for question in asi_qmnli_list if "B(G)" in question.__doc__]
asi_bi_qmnli = [question for question in asi_qmnli_list if "B(I)" in question.__doc__]
asi_bp_qmnli = [question for question in asi_qmnli_list if "B(P)" in question.__doc__]
asi_h_qmnli = [question for question in asi_qmnli_list if "H" in question.__doc__]



"""
Summary of Changes Made to ASI Questionnaire

ASIQ12:
    - Fixed modal verb: "A man must has" → "A man must have"

ASIQ17:
    - Fixed typo in descriptor Original field: '{hould be set' → 'should be set'

ASIQ22:
    - Fixed singular/plural: "A women has" → "A woman has"
"""
